#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "math/Complex.h"

#define SCREEN_WIDTH    1280
#define SCREEN_HEIGHT   720
#define MAX_ITERATIONS  1000
#define TOLERANCE       1e-6

// the function
Complex f(Complex z)
{
    // z^3 - 1
    static Complex one(1, 0);
    return z * z * z - one;
}

// derivative of the function
Complex df(Complex z)
{
    // 3 * z^2
    return (z * z) * 3;
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Nova fractal 1", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    static double zoom = 4 / (double)SCREEN_HEIGHT;
    static double centerX = (double)SCREEN_WIDTH  / 2.0;
    static double centerY = (double)SCREEN_HEIGHT / 2.0;

    // draw the fractal
    for (int y = 0; y < SCREEN_HEIGHT; y++)
    {
        for (int x = 0; x < SCREEN_WIDTH; x++)
        {
            // convert the pixel pos to a complex number
            Complex z;
            z.re = (x - centerX) * zoom;
            z.im = (y - centerY) * zoom;

            // iterate
            int iter;
            Complex a(1, 0);
            Complex c(0.3, 0.3);

            for (iter = 0; iter < MAX_ITERATIONS; iter++)
            {
                Complex oldZ = z;
                z = z - a * f(z) / df(z) + c;

                Complex delta = z - oldZ;
                double distance = delta.squareMag();

                if (distance < TOLERANCE)
                    break;
            }

            // draw the pixel
            Color color;

            float angle = z.arg();
            color.r = 127 * (1 + sin(angle));
            color.g = 127 * (1 + sin(angle + 2 * M_PI / 3));
            color.b = 127 * (1 + sin(angle + 4 * M_PI / 3));

            float coef = 1 - log(iter + 1) / 5;

            if (coef < 0)
                coef = 0;

            color.r *= coef;
            color.g *= coef;
            color.b *= coef;

            gfx.setPixel(x, y, color);
        }

        gfx.render();
        sys.processEvents();

        if (sys.isQuitRequested() == true)
            break;
    }

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        sys.processEvents();
        sys.wait(16);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
